package shooting1;
import java.awt.*;
import java.awt.event.*;


/**
 * Game object management class
 * Holds the instances of player, bullet, enemy classed
 * also manages interaction of objects(e.g. collision)
*/
public class ObjectPool
{
	
	/**
	 * Array to store enemy objects
	 */	
	static Enemy[] enemy;
	/**
	 * Array to store enemy bullets objects
	 */
	static Bullet[] bullet;
	/**
	 * Array to store player's bullets
	 */
	static MyBullet[] mybullet;
	/**
	 * Array to store explosion partivle obejects
	 */
	static Particle[] particle;

	/**
	 * Player object instance
	 */
	Player player;
	
	//Constants
	static final int DIST_PLAYER_TO_BULLET = 8;
	static final int DIST_PLAYER_TO_ENEMY = 16;
	static final int DIST_ENEMY_TO_MYBULLET = 16;
	
	//Max Constants
	static final int BULLET_MAX = 100;
	static final int ENEMY_MAX = 100;
	static final int PARTICLE_MAX = 100;
	static final int MYBULLET_MAX = 5;
	

	/**
	 * COnstracter
	 */
	ObjectPool()
	{
		//Create player object
		player = new Player(250, 400, 4);
		player.active = true;
		
		//get array for storing enemy bullets objects
		bullet = new Bullet[BULLET_MAX];
		for(int i = 0; i < bullet.length; i++)
		{
				bullet[i] = new Bullet();
		}

		//get array for storing enemy objects
		enemy = new Enemy[ENEMY_MAX];
		for(int i = 0; i < enemy.length; i++)
		{
				enemy[i] = new Enemy(player);
		}
		
		//get array for storing player's bullet objects
		mybullet = new MyBullet[MYBULLET_MAX];
		for(int i = 0; i < mybullet.length; i++)
		{
				mybullet[i] = new MyBullet();
		}

		//get array forn storing explosion particle objects
		particle = new Particle[PARTICLE_MAX];
		for(int i = 0; i < particle.length; i++)
		{
				particle[i] = new Particle();
		}
	}

	/**
	 * Graphics management
	 * Draw all objects on screen
	 */
	public void drawAll(Graphics g)
	{
        doGameObjects(g, bullet);
        doGameObjects(g, enemy);
        doGameObjects(g, mybullet);
        doGameObjects(g, particle);
		player.draw(g);
	}

	/**
	 * Draw only instances that are activated
	 * refering to each array
	 */
    public void doGameObjects(Graphics g, GameObject[] objary)
    {
        for (int i = 0; i < objary.length; i++) {
			
            if (objary[i].active == true)
			{
                objary[i].move();
                objary[i].draw(g);				
            }
        }
    }
    
    /**
     * Create Bullet object, initialize or reuse
     * @param ix x coordinate
     * @param iy y coordinate
     * @param idirection direction
     * @param ispeed speed
     * @return Bullet ID?ireturns -1 when there is no space?j
     */
	public static int newBullet(double ix, double iy, double idirection, double ispeed)
	{
		for (int i = 0; i < BULLET_MAX; i++)
		{
			if ((bullet[i].active) == false)
			{
				bullet[i].activate(ix, iy, idirection, ispeed);
				return i;
			}
		}
		return -1;		//not found
	}

    /**
     * Create Enemy object, initialize or reuse
     * @param ix x coordinate
     * @param iy y coordinate
     * @return Enemy ID?ireturns -1 when there is no space?j
     */
	public static int newEnemy(double ix, double iy)
	{
		for (int i = 0; i < ENEMY_MAX; i++)
		{
			if ((enemy[i].active) == false)
			{
				enemy[i].activate(ix, iy);
				return i;
			}
		}
		return -1;		//not found
	}

    /**
     * Create Player Bullet object, initialize or reuse
     * @param ix x coordinate
     * @param iy y coordinate
	 * @return Player Bullet ID (returns negative one when there is no space)
     */
	public static int newMyBullets(double ix, double iy)
	{
		for (int i = 0; i < MYBULLET_MAX; i++)
		{
			if ((mybullet[i].active) == false)
			{
				mybullet[i].activate(ix, iy);
				return i;
			}
		}
		return -1;		//not found
	}

    /**
     * Create Explosion particle object, initialize or reuse
     * @param ix x coordinate
     * @param iy y coordinate
     * @param idirection direction
     * @param ispeed speed
     * @return Explosion particle ID?ireturns -1 when there is no space?j
     */
	public static int newParticle(double ix, double iy, double idirection, double ispeed)
	{
		for (int i = 0; i < PARTICLE_MAX; i++)
		{
			if ((particle[i].active) == false)
			{
				particle[i].activate(ix, iy, idirection, ispeed);
				return i;
			}
		}
		return -1;		//not found
	}

	/**
	 * Player shooting
	 */
	public void shotPlayer()
	{
		//Only available when player object is active
		if (player.active)
		{
			newMyBullets(player.x, player.y);
		}
	}

	/**
	 * Player movement management
	 * @param keyinput key input instance
	 */
	public void movePlayer(KeyInput keyinput)
	{
		player.move(keyinput.getXDirection(), keyinput.getYDirection());
	}
	
	/**
	 * Returns distance between two objects
	 * @param ga game object a
	 * @param gb game object b
	 * @return distance
	 */
	public double getDistance(GameObject ga, GameObject gb)
	{
		//Pythagorean theorem
		double Xdiff = Math.abs(ga.x - gb.x);
		double Ydiff = Math.abs(ga.y - gb.y);
		return Math.sqrt(Math.pow(Xdiff,2) + Math.pow(Ydiff,2));
	}

	/**
	 * collision judgment
	 */
	public void getColision()
	{
		//enemy bullets and player
        for (int i = 0; i < bullet.length; i++) {
			if ((bullet[i].active)&&(player.active))
			{
				//Judgment
				if (getDistance(player, bullet[i]) < DIST_PLAYER_TO_BULLET)
				{
					//Player disappears
					player.active = false;

					//Create explosion
					for (int j = 0; j < 360; j += 20)
					{
						newParticle(player.x, player.y, j, 2);
					}

					//Bullet disappears
					bullet[i].active = false;
				}
			}
        }

		//Player bullet and enemy
        for (int i = 0; i < enemy.length; i++) {
			if (enemy[i].active == true)
			{
				for (int j = 0; j < mybullet.length; j++)
				{
					if (mybullet[j].active == true)
					{
						//judgment
						if (getDistance(enemy[i], mybullet[j]) < DIST_ENEMY_TO_MYBULLET)
						{
							newParticle(mybullet[j].x, mybullet[j].y, 270, 2);
							//Decrease enemy hit point
							enemy[i].hit();
							//Bullet disappears
							mybullet[j].active = false;
						}
					}
				}
			}
        }

		//Enemy and player
        for (int i = 0; i < enemy.length; i++) {
			if ((enemy[i].active)&&(player.active))
			{
				//judgment
				if (getDistance(player, enemy[i]) < DIST_PLAYER_TO_ENEMY)
				{
					//player disappears
					player.active = false;

					//Create explosion
					for (int j = 0; j < 360; j += 20)
					{
						newParticle(player.x, player.y, j, 2);
					}
					//Enemy does not diosppear
				}
			}
        }

	}
	
	/**
	 * Game over judge
	 * @return returns true when ga,e over
	 */
	public boolean isGameover()
	{
		return !player.active;
	}
	
	
	
}
